@@ -55,7 +55,8 @@ class NTBackup(Archiver):
         self.type=None
         self.target=None
         self.target_dir=None
-        
+        self.target_tape=None
+
         systemroot=os.environ.get('SystemRoot', os.environ.get('windir', 'C:\\windows'))
         self.program_exe=os.path.join(systemroot, 'system32', 'ntbackup.exe')
 
@@ -136,12 +137,17 @@ class NTBackup(Archiver):
             else:
                 self.type, self.target=self.destination.match(now, self.night_shift)
                 if self.type!=None and self.type!='none':
-                    self.target_dir=os.path.dirname(self.target)
-                    if not os.path.isdir(self.target_dir):
-                        try:
-                            os.makedirs(self.target_dir)
-                        except WindowsError, e:
-                            errors['destination']='directory not found: %s (%s)' % (self.target_dir, WindowsErrorDecode(e))
+                    if self.target[:7].lower() in ('tape://', 'tape:\\'):
+                        self.target_dir=None
+                        self.target_tape=self.target[7:]
+                    else:
+                        self.target_tape=None
+                        self.target_dir=os.path.dirname(self.target)
+                        if not os.path.isdir(self.target_dir):
+                            try:
+                                os.makedirs(self.target_dir)
+                            except WindowsError, e:
+                                errors['destination']='directory not found: %s (%s)' % (self.target_dir, WindowsErrorDecode(e))
 
         options=job.get('options', '')
         try:
@@ -160,8 +166,12 @@ class NTBackup(Archiver):
                 
                 # ntbackup backup "@m:\asx\src\magik\job1.bks" /a /d "Set created 14/11/2009 at 2:29" 
                 # /v:no /r:no /rs:no /hc:off /m normal /j "backup" /l:s /f "s:\Backup.bkf"
-                self.cmd_args=[ self.program_exe, 'backup', self.selection, '/J', job['name'], '/M', self.type, '/F', self.target, '/rs:no', ]
-                if job.get('description', None): args.extend(['/d', job.get('description')])
+                self.cmd_args=[ self.program_exe, 'backup', self.selection, '/J', job['name'], '/M', self.type, '/rs:no', ]
+                if self.target_dir: 
+                    self.cmd_args.extend(['/F', self.target])
+                if self.target_tape: 
+                    self.cmd_args.extend(['/P', self.target_tape])
+                if job.get('description', None): self.cmd_args.extend(['/d', job.get('description')])
                 self.cmd_args.append('/v:'+self.boolean[self.verify])
                 self.cmd_args.append('/r:'+self.boolean[self.restricted])
                 if self.snap:
@@ -170,6 +180,10 @@ class NTBackup(Archiver):
                 self.cmd_args.append('/l:s') # logging [s]ummary
                 self.cmd_args.extend(self.options)
                 # '/hc:off'
+                
+                # when tape is used, /um must be added at THE END of the command line.
+                if self.target_tape: 
+                    self.cmd_args.extend(['/um', ])
             
                 extra+='cmdline=%s\n' % '  '.join(self.cmd_args)
     
@@ -188,6 +202,9 @@ class NTBackup(Archiver):
             process=subprocess.Popen(param_encode(self.cmd_args, manager.default_encoding), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             std_out, std_err=process.communicate()
             end=int(time.time())
+        except UnicodeEncodeError, e:
+            st=' '.join(param_encode(self.cmd_args, manager.default_encoding, 'backslashreplace'))
+            raise RuntimeError, 'Unicode error, avoid the use of non us-ascii characters in your .ini file: '+st
         except WindowsError:
             log.error('command: %r', self.cmd_args)
             if self.is64bits: 
@@ -226,7 +243,7 @@ class NTBackup(Archiver):
         status+='exit_code=%r\r\n' % process.returncode
         status+='start=%s\r\n' % time.ctime(start)
         status+='end=%s\r\n' % time.ctime(end)
-        status+='logfile=%s\r\n' % logfile
+        status+='logfile=%s\r\n' % logfile.decode(manager.default_encoding, 'backslashreplace')
         
         status+='start_epoch=%d\r\n' % start
         status+='end_epoch=%d\r\n' % end
@@ -234,27 +251,28 @@ class NTBackup(Archiver):
         status+='type=%s\r\n' % self.type
         status+='target=%s\r\n' % self.target
         
-        # retrieve info about the target file
-        try:
-            stat=os.stat(self.target)
-        except Exception, e:
-            log.error('checking target file: %s', WindowsErrorDecode(e))
-            status+='target_stat_error=%s\r\n' % WindowsErrorDecode(e)
-        else:
-            status+='target_size=%d\r\n' % stat.st_size
-            status+='target_mtime_epoch=%d\r\n' % stat.st_mtime
-            status+='target_mtime=%s\r\n' % time.ctime(stat.st_mtime)
-        
-        # retrieve info about the target directory
         dir_out=None
-        try:
-            total_free_bytes=free_space(self.target_dir, log)
-            dir_out, _total_size=windows_list_dir(self.target_dir, log)        
-        except Exception, e:
-            log.error('checking target directory: %s', WindowsErrorDecode(e))
-            status+='target_directory_error=%s\r\n' % WindowsErrorDecode(e)
-        else:
-            status+='target_free_space=%d\r\n' % total_free_bytes
+        if self.target_dir:
+            # retrieve info about the target file
+            try:
+                stat=os.stat(self.target)
+            except Exception, e:
+                log.error('checking target file: %s', WindowsErrorDecode(e))
+                status+='target_stat_error=%s\r\n' % WindowsErrorDecode(e)
+            else:
+                status+='target_size=%d\r\n' % stat.st_size
+                status+='target_mtime_epoch=%d\r\n' % stat.st_mtime
+                status+='target_mtime=%s\r\n' % time.ctime(stat.st_mtime)
+        
+            # retrieve info about the target directory
+            try:
+                total_free_bytes=free_space(self.target_dir, log)
+                dir_out, _total_size=windows_list_dir(self.target_dir, log)        
+            except Exception, e:
+                log.error('checking target directory: %s', WindowsErrorDecode(e))
+                status+='target_directory_error=%s\r\n' % WindowsErrorDecode(e)
+            else:
+                status+='target_free_space=%d\r\n' % total_free_bytes
 
         status+='cmdline=%s\r\n' % ' '.join(self.cmd_args)
 
